@@ -19,9 +19,8 @@ use crate::ast::{Expr, Statement, Value, VarRef, VarType};
 use crate::eval;
 use crate::parser::{self, Parser};
 use crate::syms::{CallError, CallableMetadata, Command, Function, Symbol, Symbols};
-use std::future::Future;
+use async_recursion::async_recursion;
 use std::io;
-use std::pin::Pin;
 use std::rc::Rc;
 
 /// Execution errors.
@@ -93,14 +92,32 @@ impl StopReason {
     }
 }
 
+/// Trait for objects that maintain state that can be reset to defaults.
+pub trait Clearable {
+    /// Resets any state held by the object to default values.  `syms` contain the symbols of the
+    /// machine before they are cleared, in case some state is held in them too.
+    fn reset_state(&self, syms: &mut Symbols);
+}
+
 /// Executes an EndBASIC program and tracks its state.
 #[derive(Default)]
 pub struct Machine {
     symbols: Symbols,
+    clearables: Vec<Box<dyn Clearable>>,
     stop_reason: Option<StopReason>,
 }
 
 impl Machine {
+    /// Registers the given clearable.
+    ///
+    /// In the common case, functions and commands hold a reference to the out-of-machine state
+    /// they interact with.  This state is invisible from here, but we may need to have access
+    /// to it to reset it as part of the `clear` operation.  In those cases, such state must be
+    /// registered via this hook.
+    pub fn add_clearable(&mut self, clearable: Box<dyn Clearable>) {
+        self.clearables.push(clearable);
+    }
+
     /// Registers the given builtin command, which must not yet be registered.
     pub fn add_command(&mut self, command: Rc<dyn Command>) {
         self.symbols.add_command(command)
@@ -113,7 +130,10 @@ impl Machine {
 
     /// Resets the state of the machine by clearing all variable.
     pub fn clear(&mut self) {
-        self.symbols.clear()
+        for clearable in self.clearables.as_slice() {
+            clearable.reset_state(&mut self.symbols);
+        }
+        self.symbols.clear();
     }
 
     /// Tells the machine to stop execution at the next statement boundary.
@@ -162,23 +182,28 @@ impl Machine {
     }
 
     /// Assigns the value of `expr` to the variable `vref`.
-    fn assign(&mut self, vref: &VarRef, expr: &Expr) -> Result<()> {
-        let value = expr.eval(&mut self.symbols)?;
-        self.symbols.set_var(&vref, value)?;
+    async fn assign(&mut self, vref: &VarRef, expr: &Expr) -> Result<()> {
+        let value = expr.eval(&mut self.symbols).await?;
+        self.symbols.set_var(vref, value)?;
         Ok(())
     }
 
     /// Assigns the value of `expr` to the array `vref` in the position `subscripts`.
-    fn assign_array(&mut self, vref: &VarRef, subscripts: &[Expr], expr: &Expr) -> Result<()> {
+    async fn assign_array(
+        &mut self,
+        vref: &VarRef,
+        subscripts: &[Expr],
+        expr: &Expr,
+    ) -> Result<()> {
         let mut ds = Vec::with_capacity(subscripts.len());
         for ss_expr in subscripts {
-            match ss_expr.eval(&mut self.symbols)? {
+            match ss_expr.eval(&mut self.symbols).await? {
                 Value::Integer(i) => ds.push(i),
                 v => return new_syntax_error(format!("Subscript {:?} must be an integer", v)),
             }
         }
 
-        let value = expr.eval(&mut self.symbols)?;
+        let value = expr.eval(&mut self.symbols).await?;
 
         match self.symbols.get_mut(vref)? {
             Some(Symbol::Array(array)) => {
@@ -192,10 +217,15 @@ impl Machine {
 
     /// Defines a new array `name` of type `subtype` with `dimensions`.  The array must not yet
     /// exist, and the name may not overlap function or variable names.
-    pub fn dim_array(&mut self, name: &str, subtype: &VarType, dimensions: &[Expr]) -> Result<()> {
+    pub async fn dim_array(
+        &mut self,
+        name: &str,
+        subtype: &VarType,
+        dimensions: &[Expr],
+    ) -> Result<()> {
         let mut ds = Vec::with_capacity(dimensions.len());
         for dim_expr in dimensions {
-            match dim_expr.eval(&mut self.symbols)? {
+            match dim_expr.eval(&mut self.symbols).await? {
                 Value::Integer(i) => {
                     if i <= 0 {
                         return new_syntax_error("Dimensions in DIM array must be positive");
@@ -212,7 +242,7 @@ impl Machine {
     /// Executes an `IF` statement.
     async fn do_if(&mut self, branches: &[(Expr, Vec<Statement>)]) -> Result<()> {
         for (expr, stmts) in branches {
-            match expr.eval(&mut self.symbols)? {
+            match expr.eval(&mut self.symbols).await? {
                 Value::Boolean(true) => {
                     for s in stmts {
                         self.exec_one(s).await?;
@@ -238,14 +268,14 @@ impl Machine {
         debug_assert!(
             iterator.ref_type() == VarType::Auto || iterator.ref_type() == VarType::Integer
         );
-        let start_value = start.eval(&mut self.symbols)?;
+        let start_value = start.eval(&mut self.symbols).await?;
         match start_value {
             Value::Integer(_) => self.symbols.set_var(iterator, start_value)?,
             _ => return new_syntax_error("FOR supports integer iteration only"),
         }
 
         loop {
-            match end.eval(&mut self.symbols)? {
+            match end.eval(&mut self.symbols).await? {
                 Value::Boolean(false) => {
                     break;
                 }
@@ -257,7 +287,7 @@ impl Machine {
                 self.exec_one(s).await?;
             }
 
-            self.assign(iterator, next)?;
+            self.assign(iterator, next).await?;
         }
         Ok(())
     }
@@ -265,7 +295,7 @@ impl Machine {
     /// Executes a `WHILE` loop.
     async fn do_while(&mut self, condition: &Expr, body: &[Statement]) -> Result<()> {
         loop {
-            match condition.eval(&mut self.symbols)? {
+            match condition.eval(&mut self.symbols).await? {
                 Value::Boolean(true) => {
                     for s in body {
                         self.exec_one(s).await?;
@@ -279,6 +309,7 @@ impl Machine {
     }
 
     /// Executes a single statement.
+    #[async_recursion(?Send)]
     async fn exec_one<'a>(&'a mut self, stmt: &'a Statement) -> Result<()> {
         if self.stop_reason.is_some() {
             return Ok(());
@@ -286,42 +317,31 @@ impl Machine {
 
         match stmt {
             Statement::ArrayAssignment(vref, subscripts, value) => {
-                self.assign_array(vref, subscripts, value)?
+                self.assign_array(vref, subscripts, value).await?
             }
-            Statement::Assignment(vref, expr) => self.assign(vref, expr)?,
+            Statement::Assignment(vref, expr) => self.assign(vref, expr).await?,
             Statement::BuiltinCall(name, args) => {
                 let cmd = match self.symbols.get(&VarRef::new(name, VarType::Auto))? {
                     Some(Symbol::Command(cmd)) => cmd.clone(),
                     Some(_) => return new_syntax_error(format!("{} is not a command", name)),
                     None => return new_syntax_error(format!("Unknown builtin {}", name)),
                 };
-                cmd.exec(&args, self)
+                cmd.exec(args, self)
                     .await
                     .map_err(|e| Error::from_call_error(cmd.metadata(), e))?;
             }
             Statement::Dim(varname, vartype) => self.symbols.dim(varname, *vartype)?,
             Statement::DimArray(varname, dimensions, subtype) => {
-                self.dim_array(varname, subtype, dimensions)?
+                self.dim_array(varname, subtype, dimensions).await?
             }
             Statement::If(branches) => {
-                // Change this to using FutureExt::boxed_local if we ever depend on the futures or
-                // futures_lite crate directly.
-                let f: Pin<Box<dyn Future<Output = Result<()>>>> = Box::pin(self.do_if(branches));
-                f.await?;
+                self.do_if(branches).await?;
             }
             Statement::For(iterator, start, end, next, body) => {
-                // Change this to using FutureExt::boxed_local if we ever depend on the futures or
-                // futures_lite crate directly.
-                let f: Pin<Box<dyn Future<Output = Result<()>>>> =
-                    Box::pin(self.do_for(iterator, start, end, next, body));
-                f.await?;
+                self.do_for(iterator, start, end, next, body).await?;
             }
             Statement::While(condition, body) => {
-                // Change this to using FutureExt::boxed_local if we ever depend on the futures or
-                // futures_lite crate directly.
-                let f: Pin<Box<dyn Future<Output = Result<()>>>> =
-                    Box::pin(self.do_while(condition, body));
-                f.await?;
+                self.do_while(condition, body).await?;
             }
         }
         Ok(())
@@ -352,18 +372,40 @@ mod tests {
     use std::cell::RefCell;
     use std::rc::Rc;
 
+    /// A clearable that tracks whether it has been called.
+    struct MockClearable {
+        cleared: Rc<RefCell<bool>>,
+    }
+
+    impl Clearable for MockClearable {
+        fn reset_state(&self, syms: &mut Symbols) {
+            // Make sure we can see the symbols before they are cleared.
+            assert!(syms.get_var(&VarRef::new("a", VarType::Boolean)).is_ok());
+            assert!(syms.get_var(&VarRef::new("b", VarType::Integer)).is_ok());
+
+            *self.cleared.borrow_mut() = true;
+        }
+    }
+
     #[test]
     fn test_clear() {
         let mut machine = Machine::default();
+
+        let cleared = Rc::from(RefCell::from(false));
+        let clearable = Box::from(MockClearable { cleared: cleared.clone() });
+        machine.add_clearable(clearable);
+
         assert_eq!(
             StopReason::Eof,
             block_on(machine.exec(&mut b"a = TRUE: b = 1".as_ref())).expect("Execution failed")
         );
         assert!(machine.get_var_as_bool("a").is_ok());
         assert!(machine.get_var_as_int("b").is_ok());
+        assert!(!*cleared.borrow());
         machine.clear();
         assert!(machine.get_var_as_bool("a").is_err());
         assert!(machine.get_var_as_int("b").is_err());
+        assert!(*cleared.borrow());
     }
 
     #[test]
